@@ -787,12 +787,13 @@ setClass("OM", representation(Name = "character", Agency="character",
 # initialize OM
 setMethod("initialize", "OM", function(.Object, Stock=NULL, Fleet=MSEtool::Generic_Fleet,
                                        Obs=MSEtool::Generic_Obs, Imp=MSEtool::Perfect_Imp,
-                                       interval=4, pstar=0.5, maxF=0.8, reps=1, nsim=48, proyears=50) {
+                                       interval=4, pstar=0.5, maxF=0.8, reps=1, nsim=48, proyears=50, docheck=TRUE) {
   if (is.null(Stock)) {
     # message("No Stock object found. Returning a blank OM object")
     
     # Check and add defaults
-    .Object <- CheckOM(.Object, msg=FALSE, stop_if_missing=FALSE)
+    if (docheck) 
+      .Object <- CheckOM(.Object, msg=FALSE, stop_if_missing=FALSE)
     return(.Object)
   }
 
@@ -1206,7 +1207,6 @@ setMethod('summary', signature="MSE", function(object, ..., silent=FALSE, Refs=N
 
 })
 
-
 # # ---- Plot Data Object -----
 # #' Plot Data object
 # #'
