@@ -92,8 +92,6 @@ popdynMICE <- function(qsx, qfracx, np, nf, nyears, nareas, maxage, Nx, VFx, Fre
     VBfx[VBfind] <- Bx[VBfind[, c(1,3,4,5)]] * VFx[VBfind[, 1:4]]
     VBcur[] <- VBfx[, , , y-1, ] # array(VBfx[,,,y-1,], c(np, nf, n_age, nareas))
     
-    
-    
     Fdist[Find] <- VBcur[Find]*MPAthisyr[Find[,c(1,2,4)]]^Spat_targ[Find[, 1:2]]
     Fdist[Find] <- Fdist[Find]/apply(Fdist,1:3,sum)[Find[,1:3]]
     Fdist[is.na(Fdist)] <- 0 # This is an NA catch for hermaphroditism
@@ -329,8 +327,19 @@ popdynOneMICE <- function(np, nf, nareas, maxage, Ncur, Bcur, SSBcur, Vcur, FMre
   Nnext <- sapply(1:np, function(p) {
     popdynOneTScpp(nareas, maxage, Ncurr = Ncur[p, , ], Zcurr = Zcur[p, , ], plusgroup = plusgroup[p])
   }, simplify = "array") %>% aperm(c(3, 1, 2)) # np x n_age x nareas
+
   Nnext[, 1, ] <- 0
   
+  # hack for 0-filled M-at-age
+  for (p in 1:np) {
+    zero.m.ind <- which(M_agecur[p,]==0)
+    if (length(zero.m.ind)>0) {
+      # set N above maxage for this stock to zero
+      Nnext[p,min(zero.m.ind)-1,] <- Nnext[p,min(zero.m.ind)-1,] + apply(Nnext[p,zero.m.ind,], 2, sum)
+      Nnext[p,zero.m.ind,] <- 0 
+    }
+  }
+  
   # Re-assign abundance due to hermaphroditism
   if (length(SexPars$Herm)) {
     Nnext[is.na(Nnext)] <- 0 # catch for NAs
