@@ -269,7 +269,7 @@ Simulate <- function(OM=MSEtool::testOM, parallel=FALSE, silent=FALSE) {
   N[SAYR] <- StockPars$R0[S] * surv[SAY] * StockPars$initdist[SAR]*StockPars$Perr_y[Sa]  # Calculate initial stock numbers
   Biomass[SAYR] <- N[SAYR] * StockPars$Wt_age[SAY]  # Calculate initial stock biomass
   SSB[SAYR] <- SSN[SAYR] * StockPars$Fec_Age[SAY]    # Calculate spawning stock biomass
-  VBiomass[SAYR] <- Biomass[SAYR] * FleetPars$V_real[SAY]  # Calculate vulnerable biomass
+  VBiomass[SAYR] <- N[SAYR] * FleetPars$Wt_age_C[SAY] * FleetPars$V_real[SAY]  # Calculate vulnerable biomass
 
   StockPars$aR <- aR
   StockPars$bR <- bR
@@ -511,7 +511,7 @@ Simulate <- function(OM=MSEtool::testOM, parallel=FALSE, silent=FALSE) {
 
   fracD <- 0.05; ntrials <- 50
   if (!is.null(control$ntrials)) ntrials <- control$ntrials
-  if (!is.null(control$ntrials)) fracD <- control$fracD
+  if (!is.null(control$fracD)) fracD <- control$fracD
 
   # If q has hit bound, re-sample depletion and try again. Tries 'ntrials' times and then alerts user
   if (length(probQ) > 0) {
@@ -1004,9 +1004,7 @@ Simulate <- function(OM=MSEtool::testOM, parallel=FALSE, silent=FALSE) {
 
   ind <- which(lapply(ObsPars, length) == nsim)
   obs <- data.frame(ObsPars[ind])
-  ind <- which(lapply(ImpPars, length) == nsim)
-  imp <- data.frame(ImpPars[ind])
-  OMPars <- data.frame(OMPars, obs, imp)
+  OMPars <- data.frame(OMPars, obs)
 
   Hist@OMPars <- OMPars
   Hist@AtAge <- list(Length=StockPars$Len_age,
@@ -1115,33 +1113,20 @@ Project <- function (Hist=NULL, MPs=NA, parallel=FALSE, silent=FALSE,
   isrunning <- snowfall::sfIsRunning()
   if (!runparallel & isrunning) snowfall::sfStop()
 
-  if (runparallel) {
+  # Don't run MPs in parallel unless specified
+  parallel_MPs <-  rep(FALSE, nMP) 
+  if (methods::is(parallel, 'list')) {
+    parallel <- parallel[parallel==TRUE]
+    ind <- match(names(parallel), MPs)
+    ind <- ind[!is.na(ind)]
+    parallel_MPs[ind] <- TRUE
+  }
+
+  if (runparallel & any(parallel_MPs)) {
     if (!isrunning) setup()
     Export_customMPs(MPs)
   }
   
-  parallel_in <- parallel
-  if (runparallel) parallel <- rep(TRUE, nMP)
-  if (!runparallel) parallel <- rep(FALSE, nMP)
-  
-  # Don't run MSEtool MPs in parallel
-  mp_ns <- sapply(MPs, find)
-  msemmp <- grep('MSEtool', mp_ns)
-  parallel[msemmp] <- FALSE
-  
-  # Don't run DLMtool MPs in parallel except LBSPR
-  dlmmp <- grep('DLMtool', mp_ns)
-  parallel[dlmmp] <- FALSE
-  if (any(as.logical(parallel_in)) )
-    parallel[grep('LBSPR', MPs)] <- TRUE
-  
-  # Manually specified MPs
-  if (methods::is(parallel_in, 'list')) {
-    parallel_in <- data.frame(parallel_in)
-    par_mps <- names(parallel_in)
-    parallel[match(par_mps, MPs)] <- unlist(parallel_in[1,])
-  }  
-  
   # ---- Set Management Interval for each MP ----
   if (length(interval) != nMP) interval <- rep(interval, nMP)[1:nMP]
   if (!all(interval == interval[1])) {
@@ -1252,7 +1237,7 @@ Project <- function (Hist=NULL, MPs=NA, parallel=FALSE, silent=FALSE,
     Data_MP <- MSElist[[mm]]
     Data_MP@Misc <- Data_Misc # add StockPars etc back to Data object
 
-    if(!silent) message(mm, "/", nMP, " Running MSE for ", MPs[mm], ifelse(parallel[mm], " in parallel", ""))
+    if(!silent) message(mm, "/", nMP, " Running MSE for", MPs[mm], ifelse(parallel_MPs[mm], "in parallel", ""))
     checkNA <- rep(0, OM@proyears) # save number of NAs
     # years management is updated
     upyrs <- seq(from=1, to=proyears, by=interval[mm])
@@ -1317,7 +1302,7 @@ Project <- function (Hist=NULL, MPs=NA, parallel=FALSE, silent=FALSE,
     # The stock at the beginning of projection period
     N_P[,,1,] <- aperm(array(unlist(NextYrN), dim=c(n_age, nareas, nsim, 1)), c(3,1,4,2))
     Biomass_P[SAYR] <- N_P[SAYR] * StockPars$Wt_age[SAY1]  # Calculate biomass
-    VBiomass_P[SAYR] <- Biomass_P[SAYR] * V_P[SAYt]  # Calculate vulnerable biomass
+    VBiomass_P[SAYR] <- N_P[SAYR] * FleetPars$Wt_age_C[SAY1] * V_P[SAYt]  # Calculate vulnerable biomass
     SSN_P[SAYR] <- N_P[SAYR] * StockPars$Mat_age[SAY1]  # Calculate spawning stock numbers
     SSB_P[SAYR] <- N_P[SAYR] * StockPars$Fec_Age[SAY1]
 
@@ -1339,7 +1324,7 @@ Project <- function (Hist=NULL, MPs=NA, parallel=FALSE, silent=FALSE,
     )
     N_P[,,y,] <- array(unlist(Ntemp), dim=c(n_age, nareas, nsim)) %>% aperm(c(3,1,2))
     Biomass_P[SAYR] <- N_P[SAYR] * StockPars$Wt_age[SAY1]  # Calculate biomass
-    VBiomass_P[SAYR] <- Biomass_P[SAYR] * V_P[SAYt]  # Calculate vulnerable biomass
+    VBiomass_P[SAYR] <- N_P[SAYR] * FleetPars$Wt_age_C[SAY1] * V_P[SAYt]  # Calculate vulnerable biomass
     SSN_P[SAYR] <- N_P[SAYR] * StockPars$Mat_age[SAY1]  # Calculate spawning stock numbers
     SSB_P[SAYR] <- N_P[SAYR] * StockPars$Fec_Age[SAY1]
     
@@ -1351,7 +1336,8 @@ Project <- function (Hist=NULL, MPs=NA, parallel=FALSE, silent=FALSE,
     Data_MP@Misc$StockPars$SSB_P <- SSB_P
     Data_MP@Misc$StockPars$VBiomass_P <- VBiomass_P
     Data_MP@Misc$StockPars$N_P <- N_P
-    runMP <- applyMP(Data=Data_MP, MPs = MPs[mm], reps = reps, silent=TRUE, parallel = parallel[mm])  # Apply MP
+    runMP <- applyMP(Data=Data_MP, MPs = MPs[mm], reps = reps, silent=TRUE, 
+                     parallel = parallel_MPs[mm])  # Apply MP
 
     MPRecs <- runMP[[1]][[1]] # MP recommendations
     Data_p <- runMP[[2]] # Data object object with saved info from MP
@@ -1524,7 +1510,7 @@ Project <- function (Hist=NULL, MPs=NA, parallel=FALSE, silent=FALSE,
       N_P[,,y,] <- array(unlist(Ntemp), dim=c(n_age, nareas, nsim)) %>% aperm(c(3,1,2))
 
       Biomass_P[SAYR] <- N_P[SAYR] * StockPars$Wt_age[SAYt]  # Calculate biomass
-      VBiomass_P[SAYR] <- Biomass_P[SAYR] * V_P[SAYt]  # Calculate vulnerable biomass
+      VBiomass_P[SAYR] <- N_P[SAYR] * FleetPars$Wt_age_C[SAYt] * V_P[SAYt]  # Calculate vulnerable biomass
       SSN_P[SAYR] <- N_P[SAYR] * StockPars$Mat_age[SAYt]  # Calculate spawning stock numbers
       SSB_P[SAYR] <- N_P[SAYR] * StockPars$Fec_Age[SAYt]  # Calculate spawning stock biomass
       
@@ -1550,7 +1536,8 @@ Project <- function (Hist=NULL, MPs=NA, parallel=FALSE, silent=FALSE,
         Data_MP@Misc$StockPars$N_P <- N_P
 
         # --- apply MP ----
-        runMP <- applyMP(Data=Data_MP, MPs = MPs[mm], reps = reps, silent=TRUE, parallel = parallel[mm])  # Apply MP
+        runMP <- applyMP(Data=Data_MP, MPs = MPs[mm], reps = reps, silent=TRUE, 
+                         parallel = parallel_MPs[mm])  # Apply MP
         MPRecs <- runMP[[1]][[1]] # MP recommendations
         Data_p <- runMP[[2]] # Data object object with saved info from MP
         Data_p@TAC <- MPRecs$TAC
@@ -1794,7 +1781,9 @@ Project <- function (Hist=NULL, MPs=NA, parallel=FALSE, silent=FALSE,
 #' @param Hist Should model stop after historical simulations? Returns an object of
 #' class 'Hist' containing all historical data
 #' @param silent Should messages be printed out to the console?
-#' @param parallel Logical. Should the MSE be run using parallel processing? See Details for more information. 
+#' @param parallel Logical or a named list. Should MPs be run using parallel processing? 
+#' For \code{runMSE}, can also be \code{"sac"} to run the entire MSE in parallel
+#' using the split-apply-combine technique. See Details for more information. 
 #' @param extended Logical. Return extended projection results?
 #' if TRUE, `MSE@Misc$extended` is a named list with extended data
 #' (including historical and projection by area), and extended version of `MSE@Hist`
@@ -1804,18 +1793,38 @@ Project <- function (Hist=NULL, MPs=NA, parallel=FALSE, silent=FALSE,
 #' @describeIn runMSE Run the Historical Simulations and Forward Projections
 #'  from an object of class `OM
 #' @details 
-#' ## Running in Parallel
-#' For simple MPs, running in parallel can actually lead to an increase in computation time, due to the overhead in sending the 
-#' information over to the cores. Consequently, the data-limied MPs in DLMtool and the reference MPs in MSEtool are not run using parallel processing. 
-#' All other MPs, including custom MPs, will be run in parallel if argument is TRUE.
-#' To individually control which MPs run in parallel, `parallel` can be a named list of logical values, e.g., `parallel=list(AvC=TRUE)`.
+#' ## Running MPs in parallel
 #' 
+#' For most MPs, running in parallel can actually lead to an increase in computation time, due to the overhead in sending the 
+#' information over to the cores. Consequently, by default the MPs will not be run in parallel if `parallel=TRUE` 
+#' (although other internal code will be run in parallel mode).
+#' 
+#' To run MPs in parallel, specify a named list with the name of the MP(s) assigned as TRUE. For example,`parallel=list(AvC=TRUE`)
+#' will run the `AvC` MP in parallel mode.
+#' 
+#' ## Split-apply-combine MSE in parallel
+#' 
+#' Additional savings in computation time can be achieved by running the entire simulation in batches. Individual simulations of the operating model 
+#' are divided into separate cores using \link{SubCpars}, `Simulate` and `Project` are applied independently for each core via `snowfall::sfClusterApplyLB`, and the 
+#' output (a list of MSE objects) is stitched back together into a single MSE object using \link{joinMSE}. 
+#' 
+#' The ideal number of cores will be determined based on the number of simulations and available cores.
+#'  
+#' There are several issues to look out for when using this split-apply-combine technique:
+#' 
+#' \itemize{
+#' \item Numerical optimization for depletion may fail in individual cores when \code{OM@cpars$qs} is not specified.
+#' \item Length bins should be specified in the operating model in \code{OM@cpars$CAL_bins}. Otherwise, length bins can vary by core and
+#' create problems when combining into a single object.
+#' \item Compared to non-parallel runs, sampled parameters in the operating model will vary despite the same value in \code{OM@seed}.
+#' \item If there is an error in individual cores or while combining the parallel output into a single Hist or MSE object, the list of output (from the cores) will be returned.
+#' }
 #'
 #' @return Functions return objects of class \linkS4class{Hist} or \linkS4class{MSE}
 #' \itemize{
 #'   \item Simulate - An object of class \linkS4class{Hist}
 #'   \item Project - An object of class \linkS4class{MSE}
-#'   \item runMSE - An object of class \linkS4class{MSE}
+#'   \item runMSE - An object of class \linkS4class{MSE} if \code{Hist = TRUE} otherwise a class \linkS4class{Hist} object
 #' }
 #' @export
 runMSE <- function(OM=MSEtool::testOM, MPs = NA, Hist=FALSE, silent=FALSE,
@@ -1844,6 +1853,11 @@ runMSE <- function(OM=MSEtool::testOM, MPs = NA, Hist=FALSE, silent=FALSE,
   # check MPs
   if (checkMPs & !Hist)
     MPs <- CheckMPs(MPs=MPs, silent=silent)
+  
+  if (is.character(parallel) && parallel == "sac") {
+    MSEout <- runMSE_sac(OM, MPs, Hist = Hist, silent = silent, extended = extended)
+    return(MSEout)
+  }
 
   if (methods::is(OM,'OM')) {
     HistSims <- Simulate(OM, parallel, silent)  
@@ -1851,7 +1865,6 @@ runMSE <- function(OM=MSEtool::testOM, MPs = NA, Hist=FALSE, silent=FALSE,
     HistSims <- OM
   }
   
-
   if (Hist) {
     if(!silent) message("Returning historical simulations")
     return(HistSims)
@@ -1874,20 +1887,87 @@ runMSE <- function(OM=MSEtool::testOM, MPs = NA, Hist=FALSE, silent=FALSE,
 
 
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
+runMSE_sac <- function(OM, MPs, Hist = FALSE, silent = FALSE, extended = FALSE) {
+  
+  if (OM@nsim <= 48) stop("OM@nsim should be greater than 48 to effectively use split-apply-combine.")
+  if (is.null(OM@cpars$CAL_bins)) warning("OM@cpars$CAL_bins was not provided which can create issues with parallel = \"sac\"")
+  ncpus <- set_parallel(TRUE)
+  
+  nsim <- OM@nsim
+  nits <- ceiling(nsim/48)
+  itsim <- rep(48,nits)
+  if (nits < ncpus) {
+    if (nits < 4) {
+      nits <- 4
+      itsim <- rep(ceiling(nsim/4), 4)
+    } else{
+      nits <- ncpus
+      itsim <- rep(ceiling(nsim/ncpus), ncpus)
+    }
+  }
+  cnt <- 1
+  while (sum(itsim) != nsim | any(itsim<2)) {
+    diff <-  nsim - sum(itsim)
+    if (diff >0) {
+      itsim[cnt] <- itsim[cnt]+1
+    }
+    if(diff < 0) {
+      itsim[cnt] <- itsim[cnt]-1
+    }
+    cnt <- cnt+1
+    if (cnt > length(itsim)) cnt <- 1
+  }
+  
+  #### Split
+  if (!silent) {
+    message("Running MSE using split-apply-combine on ", length(itsim), " cores with these number of simulations:\n",
+            paste0(itsim, collapse = ", "))
+  }
+  sims <- lapply(1:length(itsim), function(i) {
+    if (i > 1) {
+      (sum(itsim[1:(i-1)]) + 1): sum(itsim[1:i])
+    } else {
+      1:itsim[i]
+    }
+  })
+  
+  #### Apply Simulate() in parallel
+  if (!silent) message("Running Simulate() in parallel..")
+  HistList <- snowfall::sfClusterApplyLB(1:nits, function(i, OM, iter) {
+    OM@seed <- OM@seed + i
+    tryCatch(Simulate(SubCpars(OM, sims = iter[[i]]), silent = TRUE), error = function(e) as.character(e))
+  }, OM = OM, iter = sims)
+  
+  # Error check Hist objects
+  HistErr <- sapply(HistList, function(x) !inherits(x, "Hist"))
+  if (any(HistErr)) {
+    warning("Returning list of Hist objects. There was an error when running Simulate() for core(s): ", paste0(c(1:length(HistErr))[HistErr], collapse = ", "))
+    return(HistList)
+  }
+  
+  # Combine Hist objects and exit
+  if (Hist) {
+    Histout <- try(joinHist(HistList), silent = TRUE)
+    if (methods::is(Histout, "try-error")) {
+      warning("Error in joinHist() for combining Hist objects. Returning list of Hist objects.")
+      Histout <- HistList
+    }
+    return(Histout)
+  }
+  
+  #### Apply Project() in parallel
+  Export_customMPs(MPs)
+  if (!silent) message("Running Project() in parallel with ", length(MPs), " MPs..")
+  MSElist <- snowfall::sfClusterApplyLB(HistList, function(i, MPs, extended) {
+    tryCatch(Project(i, MPs = MPs, parallel = FALSE, silent = TRUE, extended = extended, checkMPs = FALSE),
+             error = function(e) as.character(e))
+  }, MPs = MPs, extended = extended)
+  
+  #### Combine MSE objects
+  MSEout <- try(joinMSE(MSElist), silent = TRUE)
+  if (methods::is(MSEout, "try-error")) {
+    warning("Error in joinMSE() for combining MSE objects. Returning list of MSE objects.")
+    MSEout <- MSElist
+  }
+  return(MSEout)
+}
